import math, random, json

from PIL import ImageDraw, Image, ImageFont
from funcs import logThis

circleDegrees = 360
circleSize = 400
lineWidth = 40

bodySize = 700
limbSize = 300
armPosition = 200

manx = (limbSize*2)+lineWidth*4
many = (circleSize+bodySize+limbSize)+lineWidth*4

letterSize = 250
textSize = 70

letterLineLength = 300
letterLineDistance = 100

gallowx, gallowy = 1200,2000
goldenRatio = 1-(1 / ((1 + 5 ** 0.5) / 2))

smolGallowx, smolGallowy = int(gallowx * (3/10)),int(gallowy * (3/10))
smolManx, smolMany = int(manx * (3/10)),int(many * (3/10))

fnt = ImageFont.truetype('resources/comic-sans.ttf', letterSize)
smolfnt = ImageFont.truetype('resources/comic-sans.ttf', textSize)

backgroundColor = (255,255,255,255)

def calcDeviance(preDev,preDevAcc,posChange,maxmin,maxAcceleration):
    devAcc = preDevAcc + random.uniform(-posChange,posChange)
    if devAcc > maxmin * maxAcceleration: devAcc = maxmin * maxAcceleration
    elif devAcc < -maxmin  * maxAcceleration: devAcc = -maxmin * maxAcceleration

    dev = preDev + devAcc
    if dev > maxmin: dev = maxmin
    elif dev < -maxmin: dev = -maxmin
    return dev, devAcc

def badCircle():
    background = Image.new("RGBA",(circleSize+(lineWidth*3),circleSize+(lineWidth*3)),color=(0,0,0,0))

    d = ImageDraw.Draw(background,"RGBA")
    middle = (circleSize+(lineWidth*3))/2
    devx = 0
    devy = 0
    devAccx = 0
    devAccy = 0
    start = random.randint(-100,-80)
    degreesAmount = circleDegrees + random.randint(-10,30)

    for degree in range(degreesAmount):
        devx, devAccx = calcDeviance(devx,devAccx,lineWidth/100,lineWidth,0.03)
        devy, devAccy = calcDeviance(devy,devAccy,lineWidth/100,lineWidth,0.03)

        x = middle + (math.cos(math.radians(degree+start)) * (circleSize/2)) - (lineWidth/2) + devx
        y = middle + (math.sin(math.radians(degree+start)) * (circleSize/2)) - (lineWidth/2) + devy

        d.ellipse([(x,y),(x+lineWidth,y+lineWidth)],fill=(0,0,0,255))

    return background

def badLine(length, rotated = False):
    if rotated:
        w, h = length+lineWidth*3, lineWidth*3
    else:
        w, h = lineWidth*3,length+lineWidth*3
    background = Image.new("RGBA",(w,h),color=(0,0,0,0))

    d = ImageDraw.Draw(background,"RGBA")
    devx = random.randint(-int(lineWidth/3),int(lineWidth/3))
    devy = 0
    devAccx = 0
    devAccy = 0

    for pixel in range(length):
        devx, devAccx = calcDeviance(devx,devAccx,lineWidth/1000,lineWidth,0.004)
        devy, devAccy = calcDeviance(devy,devAccy,lineWidth/1000,lineWidth,0.004)

        if rotated:
            x = lineWidth + pixel + devx
            y = lineWidth + devy
        else:
            x = lineWidth + devx
            y = lineWidth + pixel + devy

        d.ellipse([(x,y),(x+lineWidth,y+lineWidth)],fill=(0,0,0,255))

    return background

def drawMan(misses):
    background = Image.new("RGBA",(manx,many),color=(0,0,0,0))

    if misses >= 1:
        head = badCircle()
        background.paste(head,(int((manx-(circleSize+(lineWidth*3)))/2),0),head)
    if misses >= 2:
        body = badLine(bodySize)
        background.paste(body,(int((manx-(lineWidth*3))/2),circleSize),body)

    if misses >= 3:
        limbs = random.sample(["rl","ll","ra","la"],min(misses-2,4))
    else: limbs = []

    for limb in limbs:
        if limb == "ra":
            limbDrawing = badLine(limbSize,True)
            rotation = random.randint(-45,45)
            xpos = int((manx-(lineWidth*3))/2)
            rotationCompensation = min(-int(math.sin(math.radians(rotation))*(limbSize+(lineWidth*3))),0)
            ypos = circleSize+armPosition + rotationCompensation
            limbDrawing = limbDrawing.rotate(rotation,expand=1)
            background.paste(limbDrawing,(xpos,ypos),limbDrawing)
        elif limb == "la":
            limbDrawing = badLine(limbSize,True)
            rotation = random.randint(-45,45)
            xpos = int((manx-(lineWidth*3))/2)-limbSize
            rotationCompensation = min(int(math.sin(math.radians(rotation))*(limbSize+(lineWidth*3))),0)
            ypos = circleSize+armPosition + rotationCompensation
            limbDrawing = limbDrawing.rotate(rotation,expand=1)
            background.paste(limbDrawing,(xpos,ypos),limbDrawing)
        elif limb == "rl":
            limbDrawing = badLine(limbSize,True)
            rotation = random.randint(-15,15)
            xpos = int((manx-(lineWidth*3))/2)-lineWidth
            ypos = circleSize+bodySize-lineWidth
            limbDrawing = limbDrawing.rotate(rotation-45,expand=1)
            background.paste(limbDrawing,(xpos,ypos),limbDrawing)
        elif limb == "ll":
            limbDrawing = badLine(limbSize,True)
            rotation = random.randint(-15,15)
            limbDrawing = limbDrawing.rotate(rotation+45,expand=1)
            xpos = int((manx-(lineWidth*3))/2)-limbDrawing.size[0]+lineWidth*3
            ypos = circleSize+bodySize
            background.paste(limbDrawing,(xpos,ypos),limbDrawing)

    return background

def badText(text, big):
    if big: font = fnt
    else: font = smolfnt
    w, h = font.getsize(text)
    img = Image.new("RGBA",(w,h),color=(0,0,0,0))
    d = ImageDraw.Draw(img,"RGBA")

    d.text((0,0),text,font=font,fill=(0,0,0,255))
    return img

def drawGallows():
    background = Image.new("RGBA",(gallowx,gallowy),color=(0,0,0,0))

    bottomLine = badLine(int(gallowx*0.75),True)
    background.paste(bottomLine,(int(gallowx*0.125),gallowy-(lineWidth*3)),bottomLine)

    lineTwo = badLine(gallowy-lineWidth*4)
    background.paste(lineTwo,(int(gallowx*(0.75*goldenRatio)),lineWidth*2),lineTwo)

    topLine = badLine(int(gallowy*0.30),True)
    background.paste(topLine,(int(gallowx*(0.75*goldenRatio))-lineWidth,lineWidth*3),topLine)

    lastLine = badLine(int(gallowy*0.125))
    background.paste(lastLine,((int(gallowx*(0.75*goldenRatio))+int(gallowy*0.30)-lineWidth),lineWidth*3),lastLine)
    return background

def drawImage(channel):
    with open("resources/games/hangmanGames.json", "r") as f:
        data = json.load(f)
    
    random.seed(data[channel]["game ID"])

    background = Image.new("RGBA",(1600,1000),color=backgroundColor)
    try:
        gallow = drawGallows()
    except:
        logThis("Error drawing gallows (error code 1711)")

    gallow = gallow.resize((smolGallowx,smolGallowy))

    try:
        man = drawMan(data[channel]["misses"])
    except:
        logThis("Error drawing stick figure (error code 1712)")

    try:
        man = man.resize((smolManx,smolMany))
    except:
        logThis("Error resizing")

    background.paste(gallow,(100,100),gallow)
    background.paste(man,(280,210),man)

    letterLines = Image.new("RGBA",((letterLineLength+letterLineDistance)*len(data[channel]["word"]),300+lineWidth*3),color=(0,0,0,0))
    for x, letter in enumerate(data[channel]["word"]):
        line = badLine(letterLineLength,True)
        letterLines.paste(line,(x*(letterLineLength+letterLineDistance),300),line)
        if data[channel]["guessed"][x]:
            letterDrawing = badText(letter,True)
            letterWidth = fnt.getsize(letter)[0]
            letterx = int(x*(letterLineLength+letterLineDistance)-(letterWidth/2)+(letterLineLength*0.5)+(lineWidth*2))
            letterLines.paste(letterDrawing,(letterx,0),letterDrawing)

    llw, llh = letterLines.size
    letterLines = letterLines.resize((int(llw*(3/10)),int(llh*(3/10))))

    background.paste(letterLines,(120,860),letterLines)

    d = ImageDraw.Draw(background,"RGBA")
    d.text((850,50),"Misses",font=smolfnt,fill=(0,0,0,255))

    background.save("resources/games/hangmanBoards/hangmanBoard"+channel+".png")
